@@ -18,21 +18,13 @@
 //!
 //! ```
 
+mod binary;
 
+use binary::Binary;
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 
-use nom::{
-    branch::alt,
-    bytes::complete::{escaped, tag, tag_no_case, take_till1, take_while_m_n},
-    character::complete::multispace0,
-    combinator::{map, peek, value as n_value},
-    error::context,
-    multi::separated_list0,
-    number::complete::double,
-    sequence::{delimited, preceded, separated_pair},
-    IResult,
-};
+use nom::{IResult, branch::alt, bytes::complete::{escaped, tag, tag_no_case, take_till1, take_while_m_n}, character::complete::{alphanumeric0, multispace0}, combinator::{map, peek, value as n_value}, error::context, multi::separated_list0, number::complete::double, sequence::{delimited, preceded, separated_pair}};
 use std::cmp::Ordering;
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
@@ -98,6 +90,28 @@ pub enum DataValue {
     /// );
     /// ```
     Tuple((Box<DataValue>, Box<DataValue>)),
+
+    /// Binary Value
+    /// 
+    /// ```
+    /// use doson::DataValue;
+    /// use doson::binary::Binary;
+    /// 
+    /// DataValue::Binary(
+    ///     Binary::build(vec![72, 101, 108, 108, 111, 32, 87, 111, 114, 108, 100])
+    /// )
+    /// 
+    /// // or
+    /// 
+    /// let from_base64: = "SGVsbG8gV29ybGQ=";
+    /// 
+    /// DataValue::Binary(
+    ///     Binary::from_base64(from_base64)
+    /// )
+    /// 
+    /// ```
+    /// 
+    Binary(binary::Binary),
 }
 
 impl std::string::ToString for DataValue {
@@ -141,6 +155,9 @@ impl std::string::ToString for DataValue {
                 let second = v.1.to_string();
 
                 format!("({},{})", first, second)
+            },
+            DataValue::Binary(v) => {
+                v.to_string()
             }
         }
     }
@@ -204,6 +221,10 @@ impl DataValue {
         }
     }
 
+    pub fn from_json(data: &str) -> Self {
+        serde_json::from_str(data).unwrap_or(Self::None)
+    }
+
     pub fn to_json(&self) -> String {
         serde_json::to_string(&self).unwrap_or(String::from("None"))
     }
@@ -286,6 +307,7 @@ impl DataValue {
             }
 
             DataValue::Tuple(tuple) => tuple.0.size() + tuple.1.size(),
+            DataValue::Binary(bin) => bin.size(),
         }
     }
 
@@ -298,6 +320,7 @@ impl DataValue {
             DataValue::List(_) => "List",
             DataValue::Dict(_) => "Dict",
             DataValue::Tuple(_) => "Tuple",
+            DataValue::Binary(_) => "Binary",
         }.to_string();
     }
 
@@ -395,6 +418,26 @@ impl ValueParser {
         )(message)
     }
 
+    fn parse_binary(message: &str) -> IResult<&str, Binary> {
+        
+        let result: (&str, &str) = context(
+            "binary", 
+            alt((
+                tag("binary!()"),
+                delimited(
+                    tag("binary!("), 
+                    take_till1(|c: char| c == '\\' || c == ')' || c.is_ascii_control()), 
+                    tag(")")
+                )
+            ))
+        )(message)?;
+
+        Ok((
+            result.0, 
+            Binary::from_b64(result.1.to_string()).unwrap_or(Binary::build(vec![]))
+        ))
+    }
+
     fn parse_number(message: &str) -> IResult<&str, f64> {
         double(message)
     }
@@ -478,6 +521,7 @@ impl ValueParser {
                     map(ValueParser::parse_list, DataValue::List),
                     map(ValueParser::parse_dict, DataValue::Dict),
                     map(ValueParser::parse_tuple, DataValue::Tuple),
+                    map(ValueParser::parse_binary, DataValue::Binary)
                 )),
                 multispace0,
             ),
@@ -488,7 +532,7 @@ impl ValueParser {
 #[cfg(test)]
 mod test {
 
-    use crate::{DataValue, ValueParser};
+    use crate::{DataValue, ValueParser, binary::Binary};
 
     #[test]
     fn list() {
@@ -524,6 +568,21 @@ mod test {
         );
     }
 
+    #[test]
+    fn binary() {
+        let message = "binary!(SGVsbG8gV29ybGQ=)";
+        assert_eq!(
+            ValueParser::parse(message),
+            Ok(
+                (
+                    "", DataValue::Binary(Binary::build(
+                        [72, 101, 108, 108, 111, 32, 87, 111, 114, 108, 100].to_vec()
+                    ))
+                )
+            )
+        )
+    }
+
     #[test]
     fn to_json() {
         let value = DataValue::List(vec![
